#!/usr/bin/perl
#
#
# Check template file for compliance with vb4.x
# Report apparent syntax errors, old-style variables and conditionals
# For valid vb 4 templates, provide a prototype set of function calls
#
# Usage : check_vb4_templates.pl
#

$Version = 0.90;

use strict;
my $templates = "../templates/templates.xml";
my $XML;
open $XML , "<", $templates or die "Unable to open $templates";

my $source = "";
while (<$XML>) {
    $source .= $_;
    chomp($source);
}
close $XML;

my %templates;

while ($source =~ m#<template name="([a-zA-Z0-9_]+)".*?version="(\d.\d.\d)".*?CDATA.(.*?)..></template>#igs) {
    $templates{$1}{'version'} = $2;
    $templates{$1}{'template'} = $3;
}

my %vars;

sub check_vb4_compliance {
    my $templatename = shift;
    my $template = shift;

    my $savetemplate = $template;

    my @errors;

    if ($template =~ /<if / or $template =~ /<else /) {
        push @errors, "Unprocessed 'if' or 'else' tags";
    }

    # Check the "condition=" fields in <vb:if>'s and <vb:elseif>'s
    my $old_conditional = 0;
    my @cond_errors;
    while ($template =~ m#condition="([^"]*)"#g) {
        my $ctest = $1;
        $ctest =~ s#(!==|===|==|!=|<=|>=)# #g;
        $ctest =~ s#[!()]##g;
        $ctest =~ s#( or | and |\s\s+)# #g;
        my @tests = split(/ /, $ctest);
        foreach my $test (@tests) {
            if ($test !~ /^\$this->registered/
                and $test !~ /^\$show/
                and $test !~ /^\$vbulletin/
                and $test !~ /^\$vboptions/
                and $test !~ /^is_browser/
                and $test !~ /^'.*'/
                and $test !~ /^-*\d+/
                and $test !~ /^>$/
                and $test !~ /^<$/
                ) {
                push @cond_errors, $test;
                $old_conditional = 1;
            }
        }
    }
    if ($old_conditional==1) {
        push @errors, "Found old conditional variable(s)";
        my $lastvar = "";
        @cond_errors = sort(@cond_errors);
        foreach my $thisvar (@cond_errors) {
            if ($thisvar ne $lastvar) {
                push @errors, "    $thisvar";
                $lastvar = $thisvar
            }
        }
    }

    # Strip any <vb:if>, <vb:else>,  </vb:if>, etc tags
    $template =~ s#condition="[^"]*"##g;
    $template =~ s#<vb:if.*?">##g;
    $template =~ s#<vb:else[^>]*>##g;
    $template =~ s#<vb:elseif.*?>##g;
    $template =~ s#<vb:comment>.*></vb:comment>##g;
    $template =~ s#</vb:if>##g;

    # Now look for untagged variables
    my $old_variable = 0;
    my @untag_errors;
    while ($template =~ m#(\$[a-zA-Z0-9\-_>]+)#g) {
        push @untag_errors, $1;
        $old_variable = 1;
    }
    if ($old_variable==1) {
        push @errors, "Found old variable(s)";
        my $lastvar = "";
        @untag_errors = sort(@untag_errors);
        foreach my $thisvar (@untag_errors) {
            if ($thisvar ne $lastvar) {
                push @errors, "    $thisvar";
                $lastvar = $thisvar
            }
        }
    }

    my $size = @errors;
    if ($size>0) {
        print "\nErrors found in $templatename\n";
        foreach (@errors) {
            print "-- $_\n";
        }
        print "\n";
    }
    else {
        print "Processed:   $templatename with no errors found\n";

        # Record all the variables used
        my $ismatch;
        $vars{$templatename} = [];

        $template = $savetemplate;
        $ismatch = 1;
        while ($ismatch==1) {
            if ($template =~ m/({vb:raw [^{}]+})/) {
                my $match = $1;
                $match =~ m/{vb:raw ([^}]+)}/;
                push @{$vars{$templatename}}, $1;
                $template =~ s/$match//g;
                $ismatch = 1;
            }
            else {
                $ismatch = 0;
            }
        }

        $template = $savetemplate;
        $ismatch = 1;
        while ($ismatch==1) {
            if ($template =~ m#(condition="([^"]*)")#g) {
                my $match = $1;
                my $ctest = $2;
                $ctest =~ s#(!==|===|==|!=|<=|>=)# #g;
                $ctest =~ s#[!()]##g;
                $ctest =~ s#( or | and |\s\s+)# #g;
                my @tests = split(/ /, $ctest);
                foreach my $test (@tests) {
                    if (
                        $test !~ /^-*\d+/
                        and $test !~ /^>$/
                        and $test !~ /^<$/
                        and $test !~ /^>=$/
                        and $test !~ /^<=$/
                        and $test !~ /^!==$/
                        and $test !~ /^===$/
                        and $test !~ /^'.*'/
                        ) {
                        my $var = $test;
                        $var =~ s/.this->registered\['([^']+)'\]/$1/g; # '
                        push @{$vars{$templatename}}, $var;
                    }
                }
                $template =~ s/$match//g;
                $ismatch = 1;
            }
            else {
                $ismatch = 0;
            }
        }
    }

}

sub print_template_vars() {

    print "Template prototypes\n\n";

    foreach my $thistemp (sort keys %vars) {
        foreach my $var (@{$vars{$thistemp}}) {
            $var =~ s/(\.|\[|->|<|>).*//;
            $var =~ s/^\$//;
        }

        print "                                                          ".$thistemp."\n";
        print "Product prototype:\n";

        print "\$templater = vB_Template::create('$thistemp');\n";
        print "\$templater->quickRegister(array(\n";

        my $lastvar = "";
LOOP:
        foreach my $var (sort @{$vars{$thistemp}}) {
            if ($var =~ /^(vbulletin|bbinfo|session|show|vboptions)/) {
                next LOOP;
            }
            if ($var ne $lastvar) {
                print "    '$var'=>\$$var, \n";
                $lastvar = $var;
            }
        }

        print "    ));\n";
        print "\$output = \$templater->render();\n";
        print "\n";

        print "System variables:\n";
        my $lastvar = "";
        foreach my $var (sort @{$vars{$thistemp}}) {
            if ($var =~ /^(vbulletin|bbinfo|session|show|vboptions)/ and $var ne $lastvar) {
                print "   $var\n";
                $lastvar = $var;
            }
        }
        print "\n";

    }
}

my $count_temp = 0;

foreach my $temp (sort {lc $a cmp lc $b} keys %templates) {
    check_vb4_compliance($temp, $templates{$temp}{'template'});
    $count_temp++;
}

print "\n\n";

print "Complete: $count_temp templates found and checked\n";

print "\n\n";

print_template_vars();

